<?php

namespace Drupal\files_url_replacer;

use Drupal\Core\File\FileUrlGenerator;
use Drupal\Core\StreamWrapper\StreamWrapperManager;
use Drupal\Core\Url;
use Symfony\Component\HttpFoundation\RequestStack;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\StreamWrapper\StreamWrapperManagerInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Language\LanguageManagerInterface;

/**
 * Replace baseurl.
 */
class CustomFileUrlGenerator extends FileUrlGenerator {

  /**
   * The config factory service.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $config;

  /**
   * The language manager service.
   *
   * @var \Drupal\Core\Language\LanguageManagerInterface
   */
  protected $languageManager;

  /**
   * {@inheritdoc}
   */
  public function __construct(
    StreamWrapperManagerInterface $stream_wrapper_manager,
    RequestStack $request_stack,
    ModuleHandlerInterface $module_handler,
    ConfigFactoryInterface $config_factory,
    LanguageManagerInterface $language_manager,
  ) {
    parent::__construct($stream_wrapper_manager, $request_stack, $module_handler);
    $this->config = $config_factory;
    $this->languageManager = $language_manager;
  }

  /**
   * {@inheritdoc}
   */
  public function generateString(string $uri) : string {
    $string = parent::generateString($uri);
    $active = &drupal_static(__FUNCTION__);
    if (!isset($active)) {
      $active = $this->config->get('files_url_replacer.settings')->get('active');
    }
    $ext = pathinfo($uri, PATHINFO_EXTENSION);
    $exclude = ['css', 'js'];

    if ($string && $active && !in_array($ext, $exclude) && StreamWrapperManager::getScheme($uri) == 'public') {
      $url = $this->config->get('files_url_replacer.settings')->get('url');
      if (!empty($url) && $this->checkIfExists($uri)) {
        $httphost = rtrim($this->requestStack->getCurrentRequest()->getSchemeAndHttpHost(), '/');
        $baseurl = rtrim(Url::fromRoute('<front>', [], ['absolute' => TRUE])->toString(), '/');
        if ($prefixes = $this->config->get('language.negotiation')->get('url.prefixes')) {
          $language = $this->languageManager->getCurrentLanguage()->getId();
          $baseurl = rtrim($baseurl, $prefixes[$language]);
          $baseurl = rtrim($baseurl, '/');
        }
        if ($httphost != $baseurl) {
          $site_path = str_replace($httphost, '', $baseurl);
          $string = str_replace($site_path, '', $string);
        }
        $string = rtrim($url, '/') . $string;
      }
    }

    return $string;
  }

  /**
   * {@inheritdoc}
   */
  public function generateAbsoluteString(string $uri) : string {
    $string = parent::generateAbsoluteString($uri);
    $active = &drupal_static(__FUNCTION__);
    if (!isset($active)) {
      $active = $this->config->get('files_url_replacer.settings')->get('active');
    }
    $ext = pathinfo($uri, PATHINFO_EXTENSION);
    $exclude = ['css', 'js'];

    if ($string && $active && !in_array($ext, $exclude) && StreamWrapperManager::getScheme($uri) == 'public') {
      $url = $this->config->get('files_url_replacer.settings')->get('url');
      if (!empty($url) && $this->checkIfExists($uri)) {
        $baseurl = rtrim(Url::fromRoute('<front>', [], ['absolute' => TRUE])->toString(), '/');
        if ($prefixes = $this->config->get('language.negotiation')->get('url.prefixes')) {
          $language = $this->languageManager->getCurrentLanguage()->getId();
          $baseurl = rtrim($baseurl, $prefixes[$language]);
          $baseurl = rtrim($baseurl, '/');
        }
        $string = str_replace($baseurl, rtrim($url, '/'), $string);
      }
    }

    return $string;
  }

  /**
   * {@inheritdoc}
   */
  private function checkIfExists(string $uri) : bool {
    $check = &drupal_static(__FUNCTION__);
    if (!isset($check)) {
      $check = $this->config->get('files_url_replacer.settings')->get('check');
    }

    if ($check == TRUE && file_exists($uri)) {
      return FALSE;
    }
    elseif ($check && str_starts_with($uri, 'public://styles/')) {
      $pieces = explode('/', $uri);
      if (!empty($pieces[2])) {
        unset($pieces[2]);
      }
      if (!empty($pieces[3])) {
        unset($pieces[3]);
      }
      if (!empty($pieces[4])) {
        unset($pieces[4]);
      }
      $uri = implode('/', $pieces);

      if (file_exists($uri)) {
        return FALSE;
      }
    }

    return TRUE;
  }

}
